// Class to hold the system state
// SystemState.h

#ifndef _SYSTEMSTATE_h
#define _SYSTEMSTATE_h

#if defined(ARDUINO) && ARDUINO >= 100
#include "arduino.h"
#else
#include "WProgram.h"
#endif
#include <ArduinoJson.h>
#include "ESPAsyncWebServer.h"

#define WIFIPORT 8081

#define ON 0
#define OFF 1
#define COOL 0
#define HEAT 1
#define TRUE true
#define FALSE false

#define DEFROST_OFFSET 30

//Non volatile memory storage
#define E_INIT				500						//EEPROM initialised flag
#define E_STARTDELAY		0						//Startup delay time
#define E_DEFROSTTIME		E_STARTDELAY + 2		//Defrost time
#define E_DEFROSTTEMP		E_DEFROSTTIME + 2		//Defrost start temperature
#define E_OPERATINGMODE		E_DEFROSTTEMP + 2		//Operating mode
#define E_ONOFF				E_OPERATINGMODE + 2		//Machine on/off
#define E_DEFROSTMAXTEMP	E_ONOFF + 2				//Temperature to stop defrost (overrides time for defrost cycle)
#define E_MAXCOMPTEMP		E_DEFROSTMAXTEMP + 2	//Maximum compressor temperature
#define E_LASTSYSTEMSTATE	E_MAXCOMPTEMP + 2		//Last state - on or off
#define E_LASTSYSTEMMODE	E_LASTSYSTEMSTATE + 2	//Last operating mode (heat/cool)
#define E_HEATTEMP			E_LASTSYSTEMMODE + 2	//Temperature for heating
#define E_COOLTEMP			E_HEATTEMP + 2			//Temperature for cooling
#define E_HYSTERISIS		E_COOLTEMP + 2			//Hysterisis setting
#define E_DEFROSTDELAY		E_HYSTERISIS + 2		//Time between defrost cycles
#define E_LASTCOMPRUN		E_DEFROSTDELAY + 2		//Time last compressor run - note 4 bytes!
#define E_USER				E_LASTCOMPRUN + 4		//Username
#define E_PASS				E_USER + 20				//Password
#define E_PORT				E_PASS + 20				//TCPIP port
#define E_WATCHDOG			E_PORT + 4				//Set to true if there was a reset by software watchdog
#define E_ROOMTEMP			E_WATCHDOG + 2			//Desired room temperature

namespace {
 const char* oState[] = { "On", "Off" };
 const char* oMode[] = { "Cool", "Heat"};
}

extern AsyncEventSource statusEvent; // event source (Server-Sent events)

class SystemState
{
public:
	SystemState();
	char www_username[20];
	char www_password[20];
	bool startupDelay = FALSE;				//Startup delay on power failure
											
	//Status of individual relays.
	bool compressorState = OFF;
	bool fanState = OFF;
	bool valveState = OFF;
	bool pumpState = OFF;

	int8_t setRoomTemperature;
	int8_t actualRoomTemperature;
	int8_t roomHumidity;
	int8_t roomDewpoint;
	bool operatingState;
	bool operatingMode;
	bool onOff;								//Machine on/off
	bool onFlag = FALSE;					//Flag an on/off operation so ignore hysterisis
	bool defrostFlag = FALSE;
	bool wifiActive = FALSE;
	long defrostCount;
	int8_t defrostStep;
	int8_t defrostStepCount;
	int8_t pressure;							//Pressure difference 
	String lastDefrost = "None";

	//System settings
	int16_t defrostRunTime;					//Time to run compressor on defrost
	int8_t defrostTemperature;
	int8_t defrostUpperTemperature;
	int8_t defrostStartDelay;
	int8_t heatTemperature;
	int8_t coolTemperature;
	int8_t hysterisis;
	int8_t maxCompressorTemperature;
	int port;
	int8_t targetTemperature;
	float humidity;
	float ambientTemperature;
	bool dataDirty = TRUE;				//Flag if status values have changed - triggers sending status json

	void setOperatingState(bool state);
	void readSettings();
	void resetParameters();
	void debugOutParameters();
	void sendSystemStatus();

private:
	String getJsonSettings();
	String getJsonStatus();
};
extern SystemState systemState;

#endif