
#include <ArduinoJson.h>
#include "HPControllerESP.h"
#include "Weather.h"

const char* weatherserver = "api.openweathermap.org";
const char* forecast3hr = "/data/2.5/forecast?id=3107677&APPID=3f5da2fcac91e83a87f8b08567c4f35f&units=metric";
const char* current = "/data/2.5/weather?id=3107677&APPID=3f5da2fcac91e83a87f8b08567c4f35f&units=metric";
const unsigned long HTTP_TIMEOUT = 10000;  // max respone time from server
WiFiClient client;

void getWeather() {
	//jsonTest();
	//return;

	int connectStatus;
	WeatherForecast newForecast;
	DynamicJsonBuffer jsonBuffer;

	getCurrentWeather(client);
	DEBUG_PRINTST("Getting Weather forecast\n[Connecting to %s ... ", weatherserver);

	if (connectStatus = client.connect(weatherserver, 80))
	{
		DEBUG_PRINTLN("connected]\n[Sending a request]");
		if (sendRequest(client, weatherserver, forecast3hr) && skipResponseHeaders(client)) {
			String json = client.readStringUntil('[');
			json = client.readStringUntil('}');
			json += "}}";
			Serial.print("json 3hr="); Serial.println(json);
			JsonObject& root = jsonBuffer.parseObject(json);

			long dt = root["dt"]; // 1492948800

			//Serial.print("3hr dt="); Serial.println(dt);
			if (dt != forecast[3].forecastTime) {
				Serial.println("Got new forecast time");
				//Move previous forecasts if we have a new forecast
				for (int i = 1; i < 7; i++) {
					forecast[i].forecastTime = forecast[i + 1].forecastTime;
					forecast[i].temperature = forecast[i + 1].temperature;
					forecast[i].pressure = forecast[i + 1].pressure;
					forecast[i].humidity = forecast[i + 1].humidity;
					forecast[i].desiredTemperature = forecast[i + 1].desiredTemperature;
				}
				forecast[3].forecastTime = root["dt"];
				JsonObject& main = root["main"];
				forecast[3].temperature = main["temp"];
				forecast[3].pressure = main["pressure"];
				forecast[3].humidity = main["humidity"];
				forecast[3].desiredTemperature = getRequiredTemperature(forecast[3].temperature);// newForecast.desiredTemperature;
			}

			for(int i=0; i<6; i++)
				json = client.readStringUntil('}');

			json = client.readStringUntil(',');
			json = client.readStringUntil('}');
			json += "}}";
			Serial.print("json 6hr="); Serial.println(json);

			JsonObject& list1 = jsonBuffer.parseObject(json);					//6 hour forecast
			long list1_dt = list1["dt"];
			//Serial.print("6hr dt="); Serial.println(list1_dt);
			if (list1_dt != forecast[5].forecastTime) {
				forecast[6].forecastTime = list1["dt"];
				JsonObject& list1_main = list1["main"];
				forecast[6].temperature = list1_main["temp"]; 
				forecast[6].pressure = list1_main["pressure"]; 
				forecast[6].humidity = list1_main["humidity"]; 
				forecast[6].desiredTemperature = getRequiredTemperature(forecast[6].temperature);//newForecast.desiredTemperature;
			}
		}
		DEBUG_PRINTLN("[Disconnected]");
		DEBUG_PRINTST("WF3 Temp:%d\n", forecast[3].temperature);
		DEBUG_PRINTST("WF3 Pressure: %d\n", forecast[3].pressure);
		DEBUG_PRINTST("WF3 Humidity: %d\n", forecast[3].humidity);
		DEBUG_PRINTST("WF3 desiredTemperature: %d\n", forecast[3].desiredTemperature);
		DEBUG_PRINTST("WF6 Temp:%d\n", forecast[6].temperature);
		DEBUG_PRINTST("WF6 Pressure: %d\n", forecast[6].pressure);
		DEBUG_PRINTST("WF6 Humidity: %d\n", forecast[6].humidity);
		DEBUG_PRINTST("WF6 desiredTemperature: %d\n", forecast[6].desiredTemperature);
	}
	else
	{
		DEBUG_PRINTST("connection failed! %d, %d\n", connectStatus, client.status());
	}
	client.stop();
	sendWeatherJson();
}

void getCurrentWeather(WiFiClient client) {
	DynamicJsonBuffer jsonBuffer;
	int connectStatus;

	DEBUG_PRINTLN("Getting current Weather\n");
	if (connectStatus =	client.connect(weatherserver, 80))
	{
		DEBUG_PRINTLN("connected]\n[Sending a request]");
		if (sendRequest(client, weatherserver, current) && skipResponseHeaders(client)) {
			String json = client.readStringUntil('\n');
			//Serial.print("json="); Serial.println(json);
			JsonObject& root = jsonBuffer.parseObject(json);
			JsonObject& main = root["main"];

			forecast[0].temperature = main["temp"];
			forecast[0].pressure = main["pressure"];
			forecast[0].humidity = main["humidity"];
			forecast[0].desiredTemperature = getRequiredTemperature(forecast[0].temperature);

			//If next setting is 0 we are on the first call so set the forward temperatures to this one
			if (forecast[1].desiredTemperature == 0) {
				DEBUG_PRINTLN("Setting weather to defaults");
				forecast[2].forecastTime = utc + 3600;
				forecast[2].temperature = forecast[0].temperature;
				forecast[2].desiredTemperature = forecast[0].desiredTemperature;
				forecast[3].forecastTime = utc + 7200;
				forecast[3].temperature = forecast[0].temperature;
				forecast[3].desiredTemperature = forecast[0].desiredTemperature;

			}
			DEBUG_PRINTST("WF Temp:%d\n", forecast[0].temperature);
			DEBUG_PRINTST("WF Pressure: %d\n", forecast[0].pressure);
			DEBUG_PRINTST("WF Humidity: %d\n", forecast[0].humidity);
			DEBUG_PRINTST("WF desiredTemperature: %d\n", forecast[0].desiredTemperature);
		}
	DEBUG_PRINTLN("[Disconnected]");
	}
	else
	{
		DEBUG_PRINTST("connection failed! %d, %d\n", connectStatus, client.status());
	}
	client.stop();
}

// Send the HTTP GET request to the server
bool sendRequest(WiFiClient client, const char* host, const char* resource) {
	Serial.print("GET ");
	Serial.println(resource);

	client.print("GET ");
	client.print(resource);
	client.println(" HTTP/1.0");
	client.print("Host: ");
	client.println(host);
	client.println("Connection: close");
	client.println();

	return true;
}

// Skip HTTP headers so that we are at the beginning of the response's body
bool skipResponseHeaders(WiFiClient client) {
	// HTTP headers end with an empty line
	char endOfHeaders[] = "\r\n\r\n";

	client.setTimeout(HTTP_TIMEOUT);
	bool ok = client.find(endOfHeaders);

	if (!ok) {
		Serial.println("No response or invalid response!");
	}

	return ok;
}

//Calculate projected temperature needed
//If in cooling mode set at 16 as not really relevant.
int getRequiredTemperature(int temperature) {
	int tempCalc;

	if (systemState.operatingMode == COOL)
		return 16;

	tempCalc = heatingRequirements[temperature] - (23 - systemState.setRoomTemperature);
	//tempCalc = (setRoomTemperature - temperature) + 16 + forecastError;
	return tempCalc;
}

//Send weather information to browser
void sendWeatherJson() {
	DynamicJsonBuffer jsonBuffer;
	JsonObject& root = jsonBuffer.createObject();
	root["currentF"] = forecast[0].temperature;
	root["currentD"] = forecast[0].desiredTemperature;
	root["time1"] = printTime(forecast[1].forecastTime);
	root["temp1F"] = forecast[1].temperature;
	root["temp1D"] = forecast[1].desiredTemperature;
	root["time2"] = printTime(forecast[2].forecastTime);
	root["temp2F"] = forecast[2].temperature;
	root["temp2D"] = forecast[2].desiredTemperature;
	root["time3"] = printTime(forecast[3].forecastTime);
	root["temp3F"] = forecast[3].temperature;
	root["temp3D"] = forecast[3].desiredTemperature;

	String jsonString;
	root.printTo(jsonString);
	weatherEvent.send(jsonString.c_str(), "wthrJson");
}
