#include "HPControllerESP.h"

void setupWebServer()
{
	// attach AsyncEventSource
	server.addHandler(&statusEvent);
	server.addHandler(&weatherEvent);
	server.addHandler(&debugEvent);

	server.on("/", HTTP_GET, [](AsyncWebServerRequest *request) {sendStatusPage(request);});
	server.on("/status", HTTP_GET, [](AsyncWebServerRequest *request) {sendStatusPage(request); });
	server.on("/refresh", HTTP_GET, [](AsyncWebServerRequest *request) {delay(1000); refreshValues(request); });
	server.on("/values", HTTP_GET, [](AsyncWebServerRequest *request) {systemState.debugOutParameters(); request->send(200); });
	server.on("/getWeather", HTTP_GET, [](AsyncWebServerRequest *request) {bSyncNTP = TRUE; request->send(200);});
	server.on("/debug", HTTP_GET, [](AsyncWebServerRequest *request) {request->send(SPIFFS, "/debug.html"); });
	server.on("/statusJson", HTTP_GET, [](AsyncWebServerRequest *request) {sendJsonStatus(request); });
	server.on("/fanOn", HTTP_GET, [](AsyncWebServerRequest *request) {setRelay(FAN, ON);  request->send(200); });
	server.on("/fanOff", HTTP_GET, [](AsyncWebServerRequest *request) {setRelay(FAN, OFF);  request->send(200); });

	server.on("/setup", HTTP_GET, [](AsyncWebServerRequest *request) {
		if (!request->authenticate(systemState.www_username, systemState.www_password)) {
			DEBUG_PRINTLN("Not authenticated");
			return request->requestAuthentication();
		}
		request->send(SPIFFS, "/setup.html"); });

	// respond to GET requests on URL /heap
	server.on("/heap", HTTP_GET, [](AsyncWebServerRequest *request) {request->send(200, "text/plain", String(ESP.getFreeHeap()));});

	server.on("/settings", HTTP_GET, [](AsyncWebServerRequest *request) {
		DEBUG_PRINTLN("settings");
		buildSettingsXML();
		request->send(200, "text/xml", strHTML);
	});

	server.on("/devReboot", HTTP_GET, [](AsyncWebServerRequest *request) {
		request->send(200, "text/plain", "Rebooting...");	
		shouldReboot = true;
	});

	server.on("/pwr", HTTP_POST, [](AsyncWebServerRequest *request) {
		if (request->args() > 0)
		{
			bool newMode = systemState.onOff;
			systemState.onOff = (int)request->arg("p").toInt();
			if (newMode != systemState.onOff)
			{
				DEBUG_PRINTST("Set power: %s\n", oState[systemState.onOff]);
				if (systemState.onOff == OFF)
				{
					millisNow = millis();
					if (systemState.defrostFlag == TRUE) {
						tmrDefrostTick.detach();
						systemState.defrostFlag = FALSE;
					}
					threadMain();							//Go turn everything off
					systemState.startupDelay = TRUE;
					DEBUG_PRINTST("Millis:%ld\n", millisNow);
				}
				systemState.onFlag = TRUE;								//Signal on so ignore hysterisis
				EEPROM.write(E_ONOFF, systemState.onOff);
				EEPROM.commit();
			}
		}
		request->send(200);
	});

	//Change operating mode - set a flag so correct sequence can be performed
	server.on("/mode", HTTP_POST, [](AsyncWebServerRequest *request) {
		if (request->args() > 0)
		{
			bool newMode = systemState.operatingMode;
			systemState.operatingMode = (int)request->arg("m").toInt();
			if (newMode != systemState.operatingMode)
			{
				DEBUG_PRINTST("mode %s\n", oMode[systemState.operatingMode]);
				millisNow = millis();
				systemState.startupDelay = TRUE;
				EEPROM.write(E_OPERATINGMODE, systemState.operatingMode);
				EEPROM.commit();
			}
		}
		//Update target temperature
		if (systemState.operatingMode == COOL)
			systemState.targetTemperature = systemState.coolTemperature;
		else
			systemState.targetTemperature = systemState.heatTemperature;
		request->send(200);
	});

	server.on("/save", HTTP_POST, [](AsyncWebServerRequest *request) {
		DEBUG_PRINTLN("save");
		if (request->args() > 0)
		{
			DEBUG_PRINTLN("args present");
			//t:defT,r : defR,u : defU,h : eatT,c : coolT,y : hyst,c : maxC,d : defD
			systemState.defrostTemperature = (int)request->arg("t").toInt();
			systemState.defrostRunTime = (int)request->arg("r").toInt();
			systemState.defrostUpperTemperature = (int)request->arg("u").toInt();
			systemState.heatTemperature = (int)request->arg("h").toInt();
			systemState.coolTemperature = (int)request->arg("c").toInt();
			systemState.hysterisis = (int)request->arg("y").toInt();
			systemState.maxCompressorTemperature = (int)request->arg("x").toInt();
			systemState.defrostStartDelay = (int)request->arg("d").toInt();
			systemState.port = request->arg("p").toInt();
			int newRoomTemperature = request->arg("b").toInt();
			//If room temperature has changed go and re calculate the future temperature requirements
			if (newRoomTemperature != systemState.setRoomTemperature) {
				systemState.setRoomTemperature = newRoomTemperature;
				for (int i = 0; i < 7; i++)
				{
					forecast[i].desiredTemperature = getRequiredTemperature(forecast[i].temperature);
				}
				EEPROM.write(E_ROOMTEMP, systemState.setRoomTemperature);
			}
			String usr = request->arg("n");
			String pwd = request->arg("a");
			EEPROM.write(E_DEFROSTTEMP, systemState.defrostTemperature + DEFROST_OFFSET);
			EEPROM.write(E_DEFROSTTIME, systemState.defrostRunTime);
			EEPROM.write(E_DEFROSTMAXTEMP, systemState.defrostUpperTemperature);
			EEPROM.write(E_HEATTEMP, systemState.heatTemperature);
			EEPROM.write(E_COOLTEMP, systemState.coolTemperature);
			EEPROM.write(E_HYSTERISIS, systemState.hysterisis);
			EEPROM.write(E_MAXCOMPTEMP, systemState.maxCompressorTemperature);
			EEPROM.write(E_DEFROSTDELAY, systemState.defrostStartDelay);
			eepromUtil.eeprom_write_int(E_PORT, systemState.port);
			usr.toCharArray(systemState.www_username, 9);
			pwd.toCharArray(systemState.www_password, 9);
			eepromUtil.eeprom_write_string(E_USER, systemState.www_username);
			eepromUtil.eeprom_write_string(E_PASS, systemState.www_password);
			EEPROM.commit();

			//Update target temperature
			if (systemState.operatingMode == COOL)
				systemState.targetTemperature = systemState.coolTemperature;
			else
				systemState.targetTemperature = systemState.heatTemperature;
		}
		request->send(200, "text/plain", "Saved settings");
		systemState.debugOutParameters();
	});

	server.on("/logout", HTTP_GET, [](AsyncWebServerRequest *request) {request->requestAuthentication();});
	// Reset all system parameters to default
	server.on("/resetParam", HTTP_GET, [](AsyncWebServerRequest *request) {
		systemState.resetParameters();
		request->send(200, "text/plain", "All parameters reset to default");
	});


	server.serveStatic("/css/style.css", SPIFFS, "/css/style.css");
	server.serveStatic("/css/table-column-options.css", SPIFFS, "/css/table-column-options.css");
	server.serveStatic("/images/bg.png", SPIFFS, "/images/bg.png");

	// Catch-All Handlers
	// Any request that can not find a Handler that canHandle it ends in the callbacks below.
	server.onNotFound(onRequest);
	server.onFileUpload(onUpload);
	server.onRequestBody(onBody);
	server.begin();
}

void sendStatusPage(AsyncWebServerRequest *request) {
	request->send(SPIFFS, "/status.html");
	systemState.dataDirty = TRUE;
}

void refreshValues(AsyncWebServerRequest *request) {
	DEBUG_PRINTLN("Refresh page data");
	request->send(200);
	systemState.dataDirty = TRUE;		//Force a data update
	//sendWeatherJson();				//Send weather information
	//sendRoomJson();					//Send room (dewpoint) information
	//systemState.sendSystemStatus();
}
void onRequest(AsyncWebServerRequest *request) {
	//Handle Unknown Request
	request->send(404);
}

void onBody(AsyncWebServerRequest *request, uint8_t *data, size_t len, size_t index, size_t total) {
	//Handle body
}

void onUpload(AsyncWebServerRequest *request, String filename, size_t index, uint8_t *data, size_t len, bool final) {
	//Handle upload
}

void onEvent(AsyncWebSocket * server, AsyncWebSocketClient * client, AwsEventType type, void * arg, uint8_t *data, size_t len) {
	//Handle WebSocket event
}

// Build the XML file with settings for setup page
void buildSettingsXML()
{
	char buf[50];
	strHTML = "<?xml version = \"1.0\" ?><inputs><defT>";
	sprintf(buf, "%d", systemState.defrostTemperature);
	strHTML += buf;
	strHTML += "</defT><defR>";
	sprintf(buf, "%d", systemState.defrostRunTime);
	strHTML += buf;
	strHTML += "</defR><defU>";
	sprintf(buf, "%d", systemState.defrostUpperTemperature);
	strHTML += buf;
	strHTML += "</defU><heatT>";
	sprintf(buf, "%d", systemState.heatTemperature);
	strHTML += buf;
	strHTML += "</heatT><coolT>";
	sprintf(buf, "%d", systemState.coolTemperature);
	strHTML += buf;
	strHTML += "</coolT><hyst>";
	sprintf(buf, "%d", systemState.hysterisis);
	strHTML += buf;
	strHTML += "</hyst><maxC>";
	sprintf(buf, "%d", systemState.maxCompressorTemperature);
	strHTML += buf;
	strHTML += "</maxC><defD>";
	sprintf(buf, "%d", systemState.defrostStartDelay);
	strHTML += buf;
	strHTML += "</defD><port>";
	sprintf(buf, "%d", systemState.port);
	strHTML += buf;
	strHTML += "</port><roomT>";
	sprintf(buf, "%d", systemState.setRoomTemperature);
	strHTML += buf;
	strHTML += "</roomT><user>";
	sprintf(buf, "%s", systemState.www_username);
	strHTML += buf;
	strHTML += "</user><mode>";
	sprintf(buf, "%s", oMode[systemState.operatingMode]);
	strHTML += buf;
	strHTML += "</mode><pass>";
	sprintf(buf, "%s", systemState.www_password);
	strHTML += buf;
	strHTML += "</pass></inputs>";
}

void setupOTAUpdate(void)
{
	// Hostname defaults to esp8266-[ChipID]
	// ArduinoOTA.setHostname("myesp8266");

	// No authentication by default
	ArduinoOTA.setPassword((const char *)"j3ss1cam");

	ArduinoOTA.onStart([]() {
		Serial.println("Start");
	});
	ArduinoOTA.onEnd([]() {
		Serial.println("\nEnd");
	});
	ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
		Serial.printf("Progress: %u%%\n", (progress / (total / 100)));
	});
	ArduinoOTA.onError([](ota_error_t error) {
		Serial.printf("Error[%u]: ", error);
		if (error == OTA_AUTH_ERROR) Serial.println("Auth Failed");
		else if (error == OTA_BEGIN_ERROR) Serial.println("Begin Failed");
		else if (error == OTA_CONNECT_ERROR) Serial.println("Connect Failed");
		else if (error == OTA_RECEIVE_ERROR) Serial.println("Receive Failed");
		else if (error == OTA_END_ERROR) Serial.println("End Failed");
	});
	ArduinoOTA.begin();
}

void sendJsonStatus(AsyncWebServerRequest *request) {
	String status = "valuesJson({\"defrostTemperature\": 25,\"heatTemperature\": -23,\"coolTemperature\": 7})";//		systemState.sendSystemStatus();
	request->send(200, "text/plain", status);
	systemState.sendSystemStatus();
}
